// Generated by CoffeeScript 1.8.0

/*
 * 微信发送消息的接口
 */

(function() {
  var RequestUrls, assert, autoReplyMessage, autoReplyTemplate, debuglog, replyImageMessage, replyNewsMessage, replyTextMessage, replyVideoMessage, replyVoiceMessage, request, sendTemplateMessage, _;

  debuglog = require("debug")("weixin_tools");

  _ = require('underscore');

  assert = require("assert");

  request = require('request');

  RequestUrls = require("../enums/request_urls");

  autoReplyTemplate = require("../enums/auto_reply_template");


  /*
   *发送模板消息
   *params
    msg =
        "touser":"o-5Zdt8pmmpmYqXbTbDUpXwx_kOk"
        "template_id":"Cx7lfTxetVWVKf77h8DRyEj93XfcazYt3faMl9Hhwl8"
        "url":"http://weixin.gamagame.cn"
        "topcolor":"#FF0000"
        "data":
           "first":
             "value":"您预约报名参加的活动，将要开始了"
             "color":"#173177"
           "keynote1":
             "value":"皇上快点群妃乱斗"
             "color":"#173177"
           "keynote2":
             "value":"今天10:00"
             "color":"#173177"
           "remark":
             "value":"欢迎进入游戏"
             "color":"#173177"
    access_token: access_token
   *return '{"errcode":40003,"errmsg":"invalid openid"}' or '{"errcode":0,"errmsg":"ok","msgid":204924419}'
   */

  sendTemplateMessage = function(msg, access_token, callback) {
    var options, url;
    assert(_.isFunction(callback), "missing callback");
    if (!((msg != null) && !_.isEmpty(msg))) {
      return callback(new Error("msg is empty"));
    }
    url = "" + RequestUrls.SEND_TEMPLATE_MESSAGE_URL + "?access_token=" + access_token;
    options = {
      url: url,
      json: true,
      method: "POST",
      body: JSON.stringify(msg)
    };
    request(options, (function(_this) {
      return function(err, res, body) {
        if (err != null) {
          return callback(err);
        }
        if (body == null) {
          return callback(new Error("invalid body"));
        }
        if (!(body.errcode === 0 && body.errmsg === 'ok')) {
          return callback(new Error("" + body.errcode + ":" + body.errmsg));
        }
        callback(null, body);
      };
    })(this));
  };

  autoReplyMessage = function(openid, type, content) {
    switch (type) {
      case autoReplyTemplate.REPLY_TYPE.text:
        return this.replyTextMessage(openid, content);
      case autoReplyTemplate.REPLY_TYPE.image:
        return this.replyImageMessage(openid, content);
      case autoReplyTemplate.REPLY_TYPE.voice:
        return this.replyVoiceMessage(openid, content);
      case autoReplyTemplate.REPLY_TYPE.video:
        return this.replyVideoMessage(openid, content);
    }
    return "";
  };

  replyTextMessage = function(openid, content) {
    var options, reply;
    options = {
      toUser: openid,
      fromUser: this.originalID,
      timestamp: this.generateTimestamp(),
      content: content
    };
    reply = _.template(autoReplyTemplate.TEXT_TP || "")(options);
    return reply;
  };

  replyImageMessage = function(openid, media_id) {
    var options, reply;
    options = {
      toUser: openid,
      fromUser: this.originalID,
      timestamp: this.generateTimestamp(),
      media_id: media_id
    };
    reply = _.template(autoReplyTemplate.IMAGE_TP || "")(options);
    return reply;
  };

  replyVoiceMessage = function(openid, media_id) {
    var options, reply;
    options = {
      toUser: openid,
      fromUser: this.originalID,
      timestamp: this.generateTimestamp(),
      media_id: media_id
    };
    reply = _.template(autoReplyTemplate.VOICE_TP || "")(options);
    return reply;
  };

  replyVideoMessage = function(openid, media_id, title, description) {
    var options, reply;
    options = {
      toUser: openid,
      fromUser: this.originalID,
      timestamp: this.generateTimestamp(),
      media_id: media_id,
      title: title || "",
      description: description || ""
    };
    reply = _.template(autoReplyTemplate.VIDEO_TP || "")(options);
    return reply;
  };

  replyNewsMessage = function(openid, items, title, description, picurl, url) {
    var articleCount, item, itemStr, options, reply, _i, _len;
    articleCount = items.length;
    itemStr = "";
    for (_i = 0, _len = items.length; _i < _len; _i++) {
      item = items[_i];
      itemStr = _.template(autoReplyTemplate.NEWS_ITEM_TP || "")(item);
    }
    options = {
      toUser: openid,
      fromUser: this.originalID,
      timestamp: this.generateTimestamp(),
      title: title || "",
      description: description || "",
      picurl: picurl || "",
      url: url || "",
      articleCount: articleCount,
      items: itemStr
    };
    reply = _.template(autoReplyTemplate.NEWS_TP || "")(options);
    return reply;
  };

  module.exports = {
    sendTemplateMessage: sendTemplateMessage,
    autoReplyMessage: autoReplyMessage,
    replyTextMessage: replyTextMessage,
    replyImageMessage: replyImageMessage,
    replyVideoMessage: replyVideoMessage,
    replyVoiceMessage: replyVoiceMessage,
    replyNewsMessage: replyNewsMessage
  };

}).call(this);
